package fr.unistra.sil.erp.back.controller.api;

import fr.unistra.sil.erp.back.model.Item;
import fr.unistra.sil.erp.back.service.CantStoreItemException;
import fr.unistra.sil.erp.back.service.StoreItemRequest;
import fr.unistra.sil.erp.back.service.StoreItemService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;

import static fr.unistra.sil.erp.back.WebMvcConfig.MAPPING_ITEMS;
import fr.unistra.sil.erp.back.model.Stock;
import fr.unistra.sil.erp.back.service.StoreStockService;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * REST API to create a new item.
 */
@Controller
public class ApiPostItemController {

    /**
     * Service for storing POJO items.
     */
    private final StoreItemService service;
    
    private final StoreStockService stockService;

    /**
     * Constructor.
     * @param service the service to store POJO items with.
     * @param stockService
     */
    public ApiPostItemController(StoreItemService service, StoreStockService stockService) {
        this.service = service;
        this.stockService = stockService;
    }

    /**
     * Creates a product.
     *
     * @param requestBody request body
     * @return a JSON response.
     * @throws ApiServerErrorException if the request could not be served.
     */
    @PostMapping(MAPPING_ITEMS)
    public ResponseEntity<Item> postItem(
            @Validated @RequestBody PostItemDTO requestBody
    ) throws ApiServerErrorException, ApiResourceNotFoundException {
        Item item;
        try {
            item = service.store(new StoreItemRequest(
                    requestBody.getName(),
                    requestBody.getPrice(),
                    requestBody.getSubscriberPrice(),
                    requestBody.getCategory()
            ));
        } catch (CantStoreItemException e) {
            throw new ApiServerErrorException(e.getMessage());
        }
        
        Stock s = new Stock(item, 0);
        try
        {
            stockService.store(s);
        } catch (SQLException ex) {
            Logger.getLogger(ApiPostItemController.class.getName()).log(Level.SEVERE, null, ex);
            throw new ApiServerErrorException(ex.getMessage());
        }

        return new ResponseEntity<>(item, HttpStatus.CREATED);
    }
}
