package fr.unistra.sil.erp.back.service;

import fr.unistra.sil.erp.back.model.RegistryEntry;
import fr.unistra.sil.erp.back.model.Stock;
import fr.unistra.sil.erp.back.repository.*;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;

/**
 *
 * @author antoine.beauvais
 */
@Service
public class UpdateStockService {
    private final IStocksRepository stocksRepository;
    private final IRegistryRepository registryRepository;

    /**
     *
     * @param repository
     * @param registryRepository
     */
    public UpdateStockService(IStocksRepository repository, IRegistryRepository registryRepository) {
        this.stocksRepository = repository;
        this.registryRepository = registryRepository;
    }

    /**
     *
     * @param request
     * @return
     * @throws CantUpdateStockException
     */
    public Stock update(UpdateStockRequest request) throws CantUpdateStockException {
        Stock stock = stocksRepository.getStockForItem(request.getItemId());
        if (stock == null)
            throw new CantUpdateStockException("Stock not found for item " + request.getItemId());
        int oldValue = stock.getQuantity();
        stock.addQuantity(request.getQuantity());
        if (oldValue == stock.getQuantity()) {
            // Skip stock update and registry update if quantity is not changed
            return stock;
        }
        try {
            stocksRepository.updateStock(stock);
        } catch (DatabaseConnectionException | DatabaseUpdateException e) {
            throw new CantUpdateStockException("Internal error");
        }
        BigDecimal totalPrice = stock.getItem().getPrice().multiply(BigDecimal.valueOf(Math.abs(request.getQuantity())));
        RegistryEntry entry = new RegistryEntry(
                null,
                request.getQuantity() > 0 ? 1 : 3,
                new Date(),
                request.getQuantity() > 0 ? 3 : 5,
                request.getQuantity() > 0 ? totalPrice : null,
                request.getQuantity() > 0 ? null : totalPrice,
                null
        );
        entry = registryRepository.store(entry);
        if (entry == null) {
            throw new CantUpdateStockException("Registry entry is not stored");
        }
        return stock;
    }
}
